/* eslint-disable unicorn/string-content */
import { describe, expect, it } from "vitest";
import { getClassName } from "./index";
import { getBuild } from "./__tests__/utils";

describe("babel-tailwind", () => {
  const compileESBuild = getBuild("main");

  it("supports ESBuild", async () => {
    const { files } = await compileESBuild({
      clsx: "emotion",
      expectFiles: 2,
      javascript: /* tsx */ `
        export function Hello() {
          return (
            <div css="text-center">
              Hello, world!
            </div>
          );
        }
      `,
    });

    const clsName = getClassName("text-center");
    expect(files.js.text).toContain(`className: "${clsName}"`);
    expect(files.css.text).toMatch(`.${clsName} {\n  text-align: center;\n}`);
  });

  it("does not remove the attribute if `preserveAttribute` is true", async () => {
    const { files } = await compileESBuild({
      clsx: "emotion",
      jsxAttributeAction: "preserve",
      expectFiles: 2,
      javascript: /* tsx */ `
        export function Hello() {
          return (
            <div css="text-center">
              Hello, world!
            </div>
          );
        }
      `,
    });

    expect(files.js.text).toContain(`css: "text-center"`);
  });

  describe('merges with existing "className" attribute', () => {
    it("string literal", async () => {
      const { files } = await compileESBuild({
        clsx: "emotion",
        expectFiles: 2,
        javascript: /* tsx */ `
          export function Hello() {
            return (
              <div className="text-center" css="text-center">
                Hello, world!
              </div>
            );
          }
      `,
      });

      const clsName = getClassName("text-center");
      expect(files.js.text).toContain(`className: "text-center ${clsName}"`);
      expect(files.css.text).toMatch(`.${clsName} {\n  text-align: center;\n}`);
    });

    it("existing function", async () => {
      const { files } = await compileESBuild({
        clsx: "emotion",
        expectFiles: 2,
        javascript: /* tsx */ `
          export function Hello() {
            return (
              <div className={({ isEntering }) => isEntering ? "enter" : "exit"} css="text-center">
                Hello, world!
              </div>
            );
          }
        `,
      });

      const clsName = getClassName("text-center");
      expect(files.js.text).toContain(
        `className: ({\n    isEntering\n  }) => _cx(isEntering ? "enter" : "exit", "${clsName}")`
      );
      expect(files.css.text).toMatch(`.${clsName} {\n  text-align: center;\n}`);
    });
  });

  it("supports custom jsxAttributeName", async () => {
    const { files } = await compileESBuild({
      clsx: "emotion",
      jsxAttributeName: "tw",
      expectFiles: 2,
      javascript: /* tsx */ `
        export function Hello() {
          return (
            <div tw="text-center">
              Hello, world!
            </div>
          );
        }
      `,
    });

    const clsName = getClassName("text-center");
    expect(files.js.text).toContain(`className: "${clsName}"`);
    expect(files.css.text).toMatch(`.${clsName} {\n  text-align: center;\n}`);
  });

  it("supports grouped array css attribute", async () => {
    const { files } = await compileESBuild({
      clsx: "emotion",
      expectFiles: 2,
      javascript: /* tsx */ `
        export function Hello() {
          return (
            <div css={["text-center", { hover: "font-semibold" }]}>
              Hello, world!
            </div>
          );
        }
      `,
    });

    const clsName = ["text-center", "hover:font-semibold"].map(getClassName);
    expect(files.js.text).toContain(`className: "${clsName.join(" ")}"`);
    expect(files.css.text).toMatch(
      [
        `.${clsName[0]} {`,
        "  text-align: center;",
        "}",
        `.${clsName[1]}:hover {`,
        "  font-weight: 600;",
        "}",
      ].join("\n")
    );
  });

  it('supports conditional expression in "css" attribute', async () => {
    const { files } = await compileESBuild({
      clsx: "emotion",
      expectFiles: 2,
      javascript: /* tsx */ `
        export function Hello({ isCenter }) {
          return (
            <div css={isCenter ? "text-center" : undefined}>
              Hello, world!
            </div>
          );
        }
      `,
    });

    const clsName = getClassName("text-center");
    expect(files.js.text).toContain(`className: isCenter ? "${clsName}" : void 0`);
    expect(files.css.text).toMatch(`.${clsName} {\n  text-align: center;\n}`);
  });
});
